      SUBROUTINE GET_THEPHI (PFTIMG, PFTS, PFT_AVGS, NROT, NANNULI, 
     1     NVIEW_MOD, SQ2, JCUT, PIXRES_MIN, PIXRES_MAX, CCPFT, ANN_LO, 
     2     ANN_HI, VAR, AVG, SQ1, 
     3     INDEX)
      
C     DETERMINES THE BEST THETA,PHI ORIENTATION
C     
C     MAY 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER
C     MODIFIED 18-AUG-1993 BY T. BAKER.
C     MODIFIED xx_APR-1996 BY DMB (Fixed bug in extended output).
C     MODIFIED 23-APR-1998 BY T. BAKER (Renamed from PFTCC_GETTP)
C     MODIFIED 14-AUG-2006, R Sinkovits, general cleanup

      IMPLICIT NONE
      
      INTEGER*4, INTENT(IN)  :: JCUT, NROT, NANNULI, NVIEW_MOD
      INTEGER*4, INTENT(IN)  :: ANN_LO, ANN_HI
      REAL*4, INTENT(IN)     :: PFTIMG((NROT/4),NANNULI)
      REAL*4, INTENT(IN)     :: PFTS((NROT/4),NANNULI,NVIEW_MOD)
      REAL*4, INTENT(IN)     :: PFT_AVGS(NVIEW_MOD)
      REAL*4, INTENT(IN)     :: PIXRES_MIN, PIXRES_MAX
      LOGICAL, INTENT(IN)    :: VAR((NROT/4),NANNULI)
      REAL*4, INTENT(IN)     :: AVG, SQ1, SQ2(NVIEW_MOD)

      INTEGER*4, INTENT(OUT) :: INDEX
      REAL*4, INTENT(OUT)    :: CCPFT

      INTEGER*4 J, ID, JN, JN_LB, JN_UB, IR, R
      REAL*4    TOT, CCOR, NPTS, DIV1, DIV2, TWOPIR, PI
      
      PI = ACOS(-1.0)

      CCPFT = -HUGE(CCPFT)
      
      DO ID = 1,NVIEW_MOD
         NPTS = 0.0
         TOT  = 0.0

         DO IR = ANN_LO,ANN_HI
            R      = MIN(NANNULI, IR + 1)
            TWOPIR = 2.0*PI*FLOAT(IR)

            jn_lb = max(1, jcut, ceiling(twopir/pixres_min))
            jn_ub = min(nrot/4-1, floor(twopir/pixres_max))

            DO JN = JN_LB, JN_UB
                  J = JN + 1
                  IF(VAR(J,R)) THEN
                     NPTS = NPTS + 1.0
                     TOT  = TOT + PFTIMG(J,R)*PFTS(J,R,ID)
                  ENDIF
            ENDDO
         ENDDO

         DIV1     = SQRT (SQ1 - NPTS*(AVG**2))
         DIV2     = SQRT (SQ2(ID)-NPTS*(PFT_AVGS(ID)**2))
         CCOR = (TOT-AVG*PFT_AVGS(ID)*NPTS)/(DIV1*DIV2)

         IF(CCOR > CCPFT) THEN
            CCPFT = CCOR
            INDEX = ID
         ENDIF
      ENDDO

      RETURN
      END SUBROUTINE GET_THEPHI
