C     *************************************************************************
C     *                                                                       *
C     *                  *****     SUBROUTINE KEY_INFO     *****              *
C     *                                                                       *
C     * ROUTINE FOR OBTAINING INFORMATION FOR RUNNING EMPFT PROGRAM.          *
C     * Modified from rtn INFO().  This rtn uses keyword input now.           *
C     *                                                                       *
C     * Originated  30-JUN-2004 BY RWA.                                       *
C     *************************************************************************
      
      SUBROUTINE KEY_INFO (NCOL, NROW, NSEC, NCOLBIN, NROWBIN, NSECBIN,
     $     NROT, NANNULI, NPIX, NFILES, NIDS, NIDS_TOT, COMPARE_MODE, 
     $     FILT_FAC)
      
      IMPLICIT NONE
      
      INCLUDE '../include/pifhead.inc'
      TYPE (PIFHEAD_T) MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD
      COMMON /PIFHEAD/   MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD
      INCLUDE '../include/infohead.inc'
      TYPE(INFOHEAD_T) INFOHEAD
      
      COMMON/PFT_COM/ MODE, BIN, VERBOSE, MAG_CEN, MAG_STEP, 
     $     MAG_NUM, MAG_NORM, XCEN_MODEL, YCEN_MODEL, SYM, DTHE, 
     $     DPHI_90, PFTRAD_LO, PFTRAD_HI, JCUT, PIXRES_MIN, PIXRES_MAX,
     $     ANN_LO, ANN_HI, VAROPT, SIGCUT, RES_MIN, RES_MAX, IDIM1,
     $     IDIM2, RADIUS, DEL_ANG, NVIEW_MOD, NTHE, NPHI, PARAM_FNAMES,
     $     IMG_FNAMES
      
      INTEGER*4 BIN, COMPARE_MODE
      INTEGER*4 NIDS_TOT, NIDS(999), NCOLBIN, NROWBIN, NSECBIN
      INTEGER*4 NCOL, NROW, NSEC, NVIEW_MOD, NTHE, NPHI, NANNULI
      INTEGER*4 I, N_POWER2, NROT, MODE, VERBOSE, NPIX
      INTEGER*4 IDIM1, IDIM2, FFT_NFAC, JCUT, ANN_LO, ANN_HI, SYM
      INTEGER*4 MAG_NUM, MAG_NORM
      INTEGER*4 NROTD, NANNULID, NVIEW_MODD, NFILES
      REAL*4    PIXSIZE, FILT_FAC(10)
      REAL*4    DEL_ANG, RADIUS , PFTRAD_LO, PFTRAD_HI, PFTRAD_STEP
      REAL*4    PI, DEL_PIXRES, RES_MIN, RES_MAX
      REAL*4    PIXRES_MIN, PIXRES_MAX, RES_TST, SIGCUT, MAG_LO, MAG_HI
      REAL*4    DANG, DTHE, DPHI_90, TEST, MAG_CEN, MAG_STEP
      REAL*4    RES_LO, RES_HI, XCEN_MODEL, YCEN_MODEL
      CHARACTER PFT_FILENAME*256, PRJ_FILENAME*256, MAP_FILENAME*256
      CHARACTER PARAM_FNAMES(999)*256, IMG_FNAMES(999)*256
      character pftrads_fname*512
      character pftres1_fname*512
      character pftres2_fname*512
      LOGICAL   VAROPT
      character(len=1)      READWRITE(4)
      
C     -------------------------------------------------------------------
      
      call copy_stdin_to_file()
      call parse_key_input(infoHead)
      
C     *************************************************************************
C     *  Input LINE #1: MODE, BIN, SYM, DANG, COMPARE_MODE, VERBOSE,          *
C     *  FILT_FAC (3I,F,2I,F)                                                 *
C     *                                                                       *
C     *  MODE  =  1 Global orientation search mode (no input origins)         *
C     *        = -1 Global mode (use stored PFTs/PRJs; no input origins)      *
C     *        =  2 Global mode (use input origins)                           *
C     *        = -2 Global mode (use stored PFTs/PRJs; use input origins)     *
C     *  BIN   = 1 Binfactor of 1 (no data compression)                       *
C     *        = 2 Binfactor of 2 (compresses 3D data by 8X; 2D data by 4X)   *
C     *  VERBOSE                                                              *
C     *     = -1 Skip calculations in global_cc. Used only for random         *
C     *          model calculations where cmp_cc is not needed                *
C     *     = 0 Minimal output (refined orientations and origins)             *
C     *     = 1 Correlation coefficients written out as functions of radius   *
C     *          and resolution in output files EMPFT.* (* = RADS,RES1,RES2)  *
C     *     = 2 gives VERBOSE=1 output and correlation coefficients for each  *
C     *             possible view                                             *
C     *  COMPARE_MODE = 0  No CTF correction                                  *
C     *        = 1  Multiply model projections by CTF                         *
C     *        = 2  Multiply RAW images by CTF (Steve Fuller method)          *
C     *        = 3  Multiply RAW images by INVERSE CTF (Xing Zhang method)    *
C     *************************************************************************
      MODE         = infoHead%input_mode
      BIN          = infoHead%bin_factor
      SYM          = infoHead%symmetry
      DANG         = infoHead%delta_theta
      COMPARE_MODE = infoHead%image_projection_compare_method
      VERBOSE      = infoHead%verbose
      FILT_FAC(1)  = infoHead%filter_factor_1
      IF(MODE.LT.-2 .OR. MODE.GT.3 .OR.MODE.EQ.0) MODE = 1
      IF(SYM.LE.0) SYM = 532
      IF(BIN.NE.2) BIN =   1    ! Current limit for this set = 2
      IF(COMPARE_MODE.LE.0 .OR. COMPARE_MODE.GT.3) COMPARE_MODE = 0
      IF(FILT_FAC(1).LE.0.0.OR.FILT_FAC(1).GT.1.0) FILT_FAC(1) = 0.1
      if(COMPARE_MODE.eq.3) then
         print *, 'FILT_FAC factor set at - ', FILT_FAC(1)
      endif
      IF(DANG.LE.0.0) DANG = 1.0
      IF(DANG.LT.0.1) DANG = 0.5
      DTHE    = DANG
      DPHI_90 = DANG
      IF(VERBOSE.NE.1 .AND. VERBOSE.NE.2 .AND. VERBOSE.NE.-1) THEN
         VERBOSE = 0
      ENDIF
C     ******************************************************************
C     *    Determine NVIEW_MOD, the number of model projection views   *
C     *     used in GLOBAL mode and check for sampling intervals.      *
C     ******************************************************************
      NTHE = 0
      NPHI = 0
      CALL GET_NVIEW_MOD (SYM, DTHE, DPHI_90, NTHE, NVIEW_MOD)
      
      PFTRAD_LO    = infoHead%pftrad_lo
      PFTRAD_HI    = infoHead%pftrad_hi
      PFTRAD_STEP  = infoHead%pftrad_step
      ANN_LO       = infoHead%annulus_low
      ANN_HI       = infoHead%annulus_high
      IF(PFTRAD_LO.LE.1.0)     PFTRAD_LO = 1.0
      IF(PFTRAD_STEP.LE.0.0) PFTRAD_STEP = 1.0
      
      RES_LO       = infoHead%resolution_low
      RES_HI       = infoHead%resolution_high
      JCUT         = infoHead%jcut
      SIGCUT       = infoHead%sigcut
      IF(JCUT.EQ.0) JCUT = 1
      IF(JCUT.LT.0) JCUT = 0
C     **************************************************************
C     *    VAROPT IS CURRENTLY NOT RECCOMMENDED BECAUSE OF BUGS    *
C     *    *****     NEED TO TEST THIS !                           *
C     **************************************************************
      VAROPT = .FALSE.
      IF(SIGCUT.GT.0.0) VAROPT = .TRUE.
      
      MAG_CEN      = infoHead%mag_cen
      MAG_STEP     = infoHead%mag_step
      MAG_NUM      = infoHead%mag_num
      MAG_NORM     = infoHead%mag_norm
      
      MAP_FILENAME = infoHead%model_filename
      IF(MODE.GT.0) THEN
         MAPHEAD%LUNUM = 1
         READWRITE(1)  = 'r'
         CALL PIF_OPEN (MAP_FILENAME, MAPHEAD%LUNUM, READWRITE) 
         CALL PIF_READ_GH (MAPHEAD)
         IF(MAPHEAD%MODE.EQ.20) THEN
            MAPHEAD%IMGNUM = 1  ! LOAD UP "IMG" #1
            CALL PIF_READ_DH (MAPHEAD)
            NCOL = MAPHEAD%NUMCOLS
            NROW = MAPHEAD%NUMROWS
            NSEC = MAPHEAD%NUMSECS
         ELSE
            PRINT *, 'Invalid PIF mode for 3D MAP'
            CALL TSBEND
         ENDIF
      ENDIF
      
      PRJ_FILENAME   = infoHead%prj_filename
      IF(PRJ_FILENAME.EQ.' ')  PRJ_FILENAME = 'PFT.PRJS'
      
      PFT_FILENAME   = infoHead%pft_filename
      IF(PFT_FILENAME.EQ.' ')  PFT_FILENAME = 'PFT.PFTS'
      
      pftrads_fname   = infoHead%pftrads_filename
      IF(pftrads_fname.EQ.' ')  pftrads_fname = 'EMPFT.RADS'
      
      pftres1_fname   = infoHead%pftres1_filename
      IF(pftres1_fname.EQ.' ')  pftres1_fname = 'EMPFT.RES1'
      
      pftres2_fname   = infoHead%pftres2_filename
      IF(pftres2_fname.EQ.' ')  pftres2_fname = 'EMPFT.RES2'
      
      IF(MODE .LE. 0) THEN
C     ***************************************************************
C     *  Need to get NCOL & NROW if reading in stored PRJs/PFTs)    *
C     ***************************************************************
         PRJHEAD%LUNUM = 3
         READWRITE(1)  = 'r'
         CALL PIF_OPEN (PRJ_FILENAME, PRJHEAD%LUNUM, READWRITE)
         CALL PIF_READ_GH (PRJHEAD)
         IF(PRJHEAD%MODE.NE.21) THEN
            PRINT *, 'Invalid PIF mode for PRJ file'
            CALL TSBEND
         ENDIF
C     ****************************************************
C     * NOTE: NSEC must be reduced by one because the    *
C     *       last entry in the PRJ file is PRJ_AVG.     *
C     ****************************************************
         PRJHEAD%IMGNUM = 1
         CALL PIF_READ_DH (PRJHEAD)
         if((prjHead%rad_lo .ne. pftrad_lo) .or.
     $        (prjHead%rad_hi .ne. pftrad_hi)) then
            print *, '*********************************************'
            print *, '* Mismatch between entered pftrad_lo/hi and *'
            print *, '* that contained in - ',prj_filename
            print *, '* Entered--> ', pftrad_lo,pftrad_hi
            print *, '* Found----> ', prjHead%rad_lo,prjHead%rad_hi
            print *, '*********************************************'
            call tsbend
         endif
         NCOL = PRJHEAD%NUMCOLS
         NROW = PRJHEAD%NUMROWS
         NSEC = PRJHEAD%NUMSECS - 1 ! Last one is the circular average
         PFTHEAD%LUNUM = 4
         READWRITE(1)  = 'r'
         CALL PIF_OPEN (PFT_FILENAME, PFTHEAD%LUNUM, READWRITE)
         CALL PIF_READ_GH (PFTHEAD)
         IF(PFTHEAD%MODE.NE.22) THEN
            PRINT *, 'Invalid PIF mode for PFT file'
            CALL TSBEND
         ENDIF
         PFTHEAD%IMGNUM = 1
         CALL PIF_READ_DH (PFTHEAD)
         if((pftHead%rad_lo .ne. pftrad_lo) .or.
     $        (pftHead%rad_hi .ne. pftrad_hi)) then
            print *, '*********************************************'
            print *, '* Mismatch between entered pftrad_lo/hi and *'
            print *, '* that contained in - ',pft_filename
            print *, '* Entered--> ', pftrad_lo,pftrad_hi
            print *, '* Found----> ', pftHead%rad_lo,pftHead%rad_hi
            print *, '*********************************************'
            call tsbend
         endif
         NROTD      = PFTHEAD%NUMCOLS
         NANNULID   = PFTHEAD%NUMROWS
         NVIEW_MODD = PFTHEAD%NUMSECS
C     *********************************************
C     *  Input line #11: Parameter data FILENAME  *
C     ********************************************
      endif
      
C     RSS - changed PFILE_INFO to PFILE_INFO_PFT in order to distinguish
C     it from the subroutine of the same name in the Commpk library.
      CALL PFILE_INFO_PFT (MODE, NCOL, NROW, NSEC, NFILES, NIDS, 
     $     NIDS_TOT, PARAM_FNAMES, IMG_FNAMES, PIXSIZE)
      
C     ************************************************************
C     *    CHECK DEFAULTS FROM INPUT LINES #2-4                  *
C     *                                                          *
C     *    CHECK VALUE OF PFTRAD_HI AGAINST SIZE OF NCOL,NROW    *
C     *     AND USE TO DETERMINE VALUE OF NROT                   *
C     ************************************************************
      RADIUS = (FLOAT(MIN(NCOL-1,NROW-1))/2.0)
      IF(PFTRAD_HI.GT.RADIUS)  PFTRAD_HI = RADIUS
      IF(PFTRAD_HI.LE.0.0)     PFTRAD_HI = RADIUS
      NANNULI = (PFTRAD_HI - PFTRAD_LO + 1.0)/PFTRAD_STEP
      pi = acos(-1.0)
      TEST     = 2*PFTRAD_HI*PI ! NEED SUFFICIENT SAMPLING AT PFTRAD_HI
      N_POWER2 = 0
      DO WHILE(TEST.GT.1)
         N_POWER2 = N_POWER2 + 1
         TEST     = TEST/2.0
      ENDDO
      NROT     = 2**N_POWER2
      DEL_ANG  = 360.0/FLOAT(NROT)
C     Why do we double the size of the fft here?????
      FFT_NFAC = 2              ! DOUBLES DEFAULT FFT SIZE FOR CC FUNCTION
      CALL FFT_SETDIM_DEF_SAME (NCOL, NROW, IDIM1, IDIM2, FFT_NFAC)
      
      IF(MODE. LE .0) THEN
C     ********************************************************
C     *    HERE IF READING IN PFT/PRJ DATA FROM DISK.        *
C     *    CHECK TO SEE THAT PFT/PRJ DATA FILES AGREE WITH   *
C     *     INPUTS FROM COMMAND BATCH JOB.                   *
C     ********************************************************
C     better check out use of nrot
C     
         IF(NROTD*4 .NE. NROT .OR. NANNULID .NE. NANNULI 
     $        .OR. NVIEW_MODD .NE. NVIEW_MOD) THEN
            PRINT *, 'NROT,NANNULI,NVIEW_MOD - ', 
     $           NROT,NANNULI,NVIEW_MOD
            PRINT *, 'NROTD,NANNULID,NVIEW_MODD - ', NROTD,NANNULID
            PRINT *,  NVIEW_MODD
            PRINT *, 'Mismatch between PFT file and batch job params'
            CALL TSBEND
         ENDIF
      ENDIF
      
      PFTRAD_STEP = 1.0
      DEL_PIXRES  = 0.025
      IF(JCUT.GE.NROT/4)   JCUT  = (NROT/4) - 1
      IF(PIXSIZE.LE.0.0) PIXSIZE = 1.0 ! PIXEL UNITS ASSUMED
      RES_TST = FLOAT(IDIM1/2)*PIXSIZE
      IF(RES_LO.LE.0.0 .OR. RES_LO.GT.RES_TST) RES_LO = RES_TST
      RES_TST = 2.0*PIXSIZE
      IF(RES_HI.LT.RES_TST) RES_HI = RES_TST
      PIXRES_MIN = RES_LO/PIXSIZE
      PIXRES_MAX = RES_HI/PIXSIZE
      RES_MIN    = FLOAT(IDIM1)/PIXRES_MIN
      RES_MAX    = FLOAT(IDIM1)/PIXRES_MAX
      
      IF(VERBOSE .GE. 1) THEN
C     ******************************************
C     *  Extra output files if VERBOSE = 1 or 2  *
C     ******************************************
         OPEN (UNIT=8, FILE=pftres1_fname)
         WRITE(8,'(2A,/,A)') 'Correlation coefficients as function ',
     $        ' of resolution (1/pixel) for unmasked data',
     $        ' (computed after X,Y origin refinement)'
         WRITE(8,'(A,10F7.3)')  ' IMG', (I*DEL_PIXRES, I=1,10)
         WRITE(8,1000)
         OPEN (UNIT=9, FILE=pftrads_fname)
         WRITE(9,'(2A,/,A)') 
     $        ' Correlation coefficients as a function',
     $        ' of radius', ' (computed after X,Y origin refinement)'
         OPEN (UNIT=10, FILE=pftres2_fname)
         WRITE(10,'(2A,/,A)') 
     $        ' Correlation coefficients as function',
     $        ' of resolution (1/pixel) for ANN_LO to ANN_HI masked',
     $        ' data (computed after X,Y origin refinement)'
         WRITE(10,'(A,10F7.3)')  ' IMG', (I*DEL_PIXRES, I=1,10)
         WRITE(10,1000)
      ENDIF
      
      IF(MAG_NUM.LE.0) MAG_NUM = 1
      IF(MOD(MAG_NUM,2).EQ.0) MAG_NUM = MAG_NUM + 1
      IF(IABS(MODE).EQ.1 .AND. MAG_CEN.LE.0.0) THEN
         MAG_CEN = 1.0
         PRINT *,'GLOBAL SEARCH mode - you have no ',
     $        'MAG values to read in'
      ENDIF
C     ******************************************************************
C     *    BEFORE EXITING THIS ROUTINE, THIS IS WHERE WE MAKE SURE     *
C     *    EVERTHING IS TREATED CORRECTLY TO ACCOMODATE BINNING.       *
C     ******************************************************************
      NCOLBIN = (NCOL/BIN) + 1
      NROWBIN = (NROW/BIN) + 1
      NSECBIN = (NSEC/BIN) + 1
      IF(MOD(NCOLBIN,2).EQ.0) NCOLBIN = NCOLBIN - 1
      IF(MOD(NROWBIN,2).EQ.0) NROWBIN = NROWBIN - 1
      IF(MOD(NSECBIN,2).EQ.0) NSECBIN = NSECBIN - 1
      NROT       = NROT/BIN
      IDIM1      = IDIM1/BIN
      IDIM2      = IDIM2/BIN
      NANNULI    = NANNULI/BIN
      NPIX       = NROT*NANNULI/4
      DEL_ANG    = DEL_ANG*FLOAT(BIN)
      ANN_LO     = ANN_LO/BIN
      ANN_HI     = ANN_HI/BIN
      RADIUS     = FLOAT(NANNULI)
      PFTRAD_LO  = PFTRAD_LO/FLOAT(BIN)
      IF(PFTRAD_LO.LT.1.0) PFTRAD_LO = 1.0
      PFTRAD_HI  = PFTRAD_HI/FLOAT(BIN)
      XCEN_MODEL = (NCOLBIN+1)/2
      YCEN_MODEL = (NROWBIN+1)/2
      IF(ANN_LO.LT.1)       ANN_LO = 0
      IF(ANN_LO.GE.NANNULI) ANN_LO = NANNULI - 1
      IF(ANN_HI.LE.0)       ANN_HI = NANNULI - 1
      IF(ANN_HI.GE.NANNULI) ANN_HI = NANNULI - 1
      WRITE(*,1001) ' EMPFT parameters'
      WRITE(*,1001) ' ---------------------------------------------'
      WRITE(*,1001) ' BIN factor          ', BIN
      WRITE(*,1001) ' Symmetry            ', SYM
      IF(MODE.EQ.3) THEN
         WRITE(*,1001) ' # model views/window', NVIEW_MOD
      ELSE
         WRITE(*,1001) ' # model views/asu   ', NVIEW_MOD
      ENDIF
      WRITE(*,1001) ' # particle images   ', NIDS_TOT
      WRITE(*,1001) ' FFT_DIM1,FFT_DIM2   ', IDIM1, IDIM2
      WRITE(*,1002) ' RES_LO,RES_HI       ', RES_LO, RES_HI
      WRITE(*,1002) ' RES_MIN,RES_MAX     ', RES_MIN, RES_MAX
      WRITE(*,1002) ' PIXRES_MIN/MAX      ', PIXRES_MIN, PIXRES_MAX
      WRITE(*,1002) ' Pixel size          ', PIXSIZE
      WRITE(*,1001) ' NANNULI             ', NANNULI
      WRITE(*,1002) ' PFTRAD_LO,PFTRAD_HI ', PFTRAD_LO, PFTRAD_HI
      WRITE(*,1001) ' ANN_LO,ANN_HI       ', ANN_LO, ANN_HI
      WRITE(*,1001) ' NROT                ', NROT
      WRITE(*,1002) ' Delta omega         ', DEL_ANG
      WRITE(*,1002) ' Model XCEN,YCEN     ', XCEN_MODEL, YCEN_MODEL
      WRITE(*,1002) ' MAG(_CEN,_STEP,_NUM)', MAG_CEN,MAG_STEP,MAG_NUM
      IF(MAG_CEN.GT.0.0) THEN
         MAG_LO = MAG_CEN - (((MAG_NUM-1)/2)*MAG_STEP)
         MAG_HI = MAG_CEN + (((MAG_NUM-1)/2)*MAG_STEP)
         WRITE(*,1002) ' MAG range(LO,HI)    ', MAG_LO, MAG_HI
      ELSE
         WRITE(*,'(2A)') ' MAG factor centered on input values from',
     $        ' data file'
      ENDIF
      WRITE(*,1001) ' ----------------------------------------------'
      RETURN
      
      PRINT *, '************************************************'
      PRINT *, '* BATCH COMMAND JOB input ERROR - STOP program *'
      CALL TSBEND
 1000 FORMAT(74('-'))
 1001 FORMAT(A,2I9)
 1002 FORMAT(A,2F9.3,I9)
      END subroutine key_info
